--[[
Name:		  LibFrameX-1.0
Author(s): 	  Azmaedus
Revision: 	  $Rev: 1 $
Created:	  01/23/2015
Description:  Library that creates a frame structure dynamically.
Dependencies: LibStub
License:	  GPLv2

	Change Log
	--------------------
	01/23/2015 - 1.0a 		Creation of addon
	01/24/2015 - 1.0a-r2	Changed the name from GJBPlayer-1.0 to LibFrameX-1.0
]]

local LIBFRAMEX = "LibFrameX-1.0"
local LIBFRAMEX_MINOR = 1 + tonumber(("$Revision: 1 $"):match("(%d+)"))

local FrameX = LibStub:NewLibrary(LIBFRAMEX, LIBFRAMEX_MINOR)
if not FrameX then return end

local defaults = {
	-- Background
	BG = {
		-- Backdrop info
		BD = {
			bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
			edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
			tile = true, tileSize = 4, edgeSize = 4,
			insets = { left = 1, right = 1, top = 1, bottom = 1 }
		},
		align = "CENTER", xPos = 0, yPos = 0, Scale = 1, w = 200,	h = 150, r = 0,	g = 0, b = 0, a = 1, br = 1, bg = 1, bb = 1, ba = 1,
	},
	--CtrlButtons = {	align = "BOTTOMCENTER",	xPos = 0, yPos = 0, Scale = 1, space = 1, w = 40, h = 20, r = 0, g = 0, b = 0, a = 1, br = 1, bg = 1, bb = 1, ba = 1 },
}

if not FrameX.frame then
	FrameX.frame = CreateFrame("Frame")
	FrameX.frame:SetWidth(defaults.BG.w)
	FrameX.frame:SetHeight(defaults.BG.h)
	FrameX.frame:SetBackdrop(defaults.BG.BD)
	FrameX.frame:SetBackdropColor(defaults.BG.r, defaults.BG.g, defaults.BG.b, defaults.BG.a)
	FrameX.frame:SetBackdropBorderColor(defaults.BG.br, defaults.BG.bg, defaults.BG.bb, defaults.BG.ba)
	FrameX.frame.pos = {}
	FrameX.frame:Show()
end

local L_FRAMEX_TITLE = "LibFrameX-1.0 Frame"
local L_FRAMEX_ERROR = "Error creating frame!"

local l = GetLocale()
if l == "frFR" then
	L_FRAMEX_TITLE = "LibFrameX-1.0 Frame"
	L_FRAMEX_ERROR = "Erreur lors de la creation de la frame!"
elseif l == "koKR" then
	-- Missing
elseif l == "deDE" then
	-- Missing
elseif l == "zhCN" then
	-- Missing
elseif l == "zhTW" then
	-- Missing
elseif l == "ruRU" then
	-- Missing
elseif l == "esES" or l == "esMX" then
	-- Missing
elseif l == "ptBR" then
	-- Missing
elseif l == "itIT" then
	-- Missing
end

-- FrameX publics
FrameX.tframes = {}

-- Returns the index of a frame by its name
function FrameX:findFrame(name)
	for i=1,table.getn(self.tframes) do
		if self.tframes[i].name == name then
			return i
		end
	end
	return nil
end

-- Function to create a sub-frame
function FrameX:genFrame(f, parent)
		--print(f.name)
	if not f then return end -- fail
	
	local fo = {}
	if parent == UIParent then
		fo = CreateFrame(f.ftype, f.name, UIParent)
	else
		fo = CreateFrame(f.ftype, f.name, parent, f.template)
	end
	
	fo.name = f.name
	
	if f.parentrgba and f.parentrgba == true and f.parentbrgba and f.parentbrgba == true and f.parentbd and f.parentbd == true then
		local rgba = {}
		local brgba = {}
		local bd = {}
		if parent and not f.root then
			local id = self:findFrame(f.parentname)
			if id then
				rgba = self.tframes[id].childrgba
				brgba = self.tframes[id].childbrgba
				bd = self.tframes[id].childbd
			else
				rgba = f.rgba
				brgba = f.brgba
				bd = f.bd
			end
		else
			rgba = f.rgba
			brgba = f.brgba
			bd = f.bd
		end
		fo:SetBackdrop(bd)
		fo:SetBackdropColor(rgba.r/255, rgba.g/255, rgba.b/255, rgba.a)
		fo:SetBackdropBorderColor(brgba.r/255, brgba.g/255, brgba.b/255, brgba.a)
	elseif f.bd then
		fo:SetBackdrop(f.bd)
		fo:SetBackdropColor(f.rgba.r/255, f.rgba.g/255, f.rgba.b/255, f.rgba.a)
		fo:SetBackdropBorderColor(f.brgba.r/255, f.brgba.g/255, f.brgba.b/255, f.brgba.a)
	end

	if f.parentdim and f.parentdim == true then
		local dim = {}
		if parent then
			local id = self:findFrame(f.parentname)
			if id then
				dim = self.tframes[id].childdim
			else
				dim = f.dim
			end
		else
			dim = f.dim
		end
		fo:SetWidth(dim.w)
		fo:SetHeight(dim.h)
	else
		fo:SetWidth(f.dim.w)
		fo:SetHeight(f.dim.h)
	end

	if f.bd then
	end
	
	if f.show then
		fo:Show()
	end
	return fo
end

--[[
Creates all sub frames by passing a table of frames to create
Structure of the tframes table:
	t = {
		{
			ftype = "type", --Frame, Button, ...
			rgba = {r = 1, g = 1, b = 1, a = 1}, -- red, green, blue, alpha 0.0 to 1.0
			dim = {w = 1, h = 1, s = 1}, -- width, height, scale
			pos = {x = 0, y = 0, a = "ANCHOR", r = "PARENT", ap = "PARENT-ANCHOR"}, -- x, y, anchor, relative to, r anchor
			bd = {}, -- BackDrop
			brgba = {r = 1, g = 1, b = 1, a = 1}, -- BackDrop : red, green, blue, alpha 0.0 to 1.0
			show = true,
			movable = {
				enabled = false, 
				drag = {
					"LeftButton",
					...
				}
			},
			clicks = {"LeftButton","RightButton","Button4"},
			scripts = {
				{event = "OnSomething", func = function(self,...)end},
				...
			},
		},
		...
	}
]]
FrameX.prev = UIParent

function FrameX:makeFrames(t)
FrameX.tframes = {}
	for i=1,#t do
		-- check if frame exists
		if t[i] then
			FrameX.tframes[i] = FrameX:genFrame(t[i], FrameX.prev)
			FrameX.prev = FrameX.tframes[i]
			FrameX.tframes[i].parentrgba = t[i].parentrgba
			FrameX.tframes[i].parentdim = t[i].parentdim
			FrameX.tframes[i].parentname = t[i].parentname
			FrameX.tframes[i].childrgba = t[i].childrgba
			FrameX.tframes[i].childrgbashaded = t[i].childrgbashaded
			FrameX.tframes[i].childbrgba = t[i].childbrgba
			FrameX.tframes[i].childbd = t[i].childbd
			FrameX.tframes[i].childdim = t[i].childdim
			FrameX.tframes[i].rgba = t[i].rgba
			FrameX.tframes[i].rgbashaded = t[i].rgbashaded
			FrameX.tframes[i].brgba = t[i].brgba
			FrameX.tframes[i].bd = t[i].bd
			FrameX.tframes[i].dim = t[i].dim
			FrameX.tframes[i].root = t[i].root
			
			FrameX.tframes[i]:SetPoint(t[i].pos.a, t[i].pos.r, t[i].pos.rp, t[i].pos.x, t[i].pos.y)
			--print(t[i].pos.a, t[i].pos.r, t[i].pos.ap, t[i].pos.x, t[i].pos.y)
			
			if t[i].text then
				FrameX.tframes[i].text = FrameX.tframes[i]:CreateFontString(nil, "ARTWORK", "NumberFont_Shadow_Small")
				FrameX.tframes[i].text:SetAllPoints(true)
				FrameX.tframes[i].text:SetJustifyH("CENTER")
				FrameX.tframes[i].text:SetJustifyV("CENTER")
				FrameX.tframes[i].text:SetTextColor(1,1,1,1)
				FrameX.tframes[i].text:SetText(t[i].text)
			end
			
			-- if frame is movable
			if t[i].movable and t[i].movable.enabled then
				FrameX.tframes[i]:SetMovable(true)
				FrameX.tframes[i]:EnableMouse(true)
				FrameX.tframes[i]:RegisterForDrag(unpack(t[i].movable.drag))
				--FrameX.tframes[i]:SetScale(t[i].dim.s)
			end

			-- register click events
			if t[i].clicks then
				FrameX.tframes[i]:SetPoint("CENTER")
				FrameX.tframes[i]:RegisterForClicks(unpack(t[i].clicks))
				--FrameX.tframes[i]:SetNormalTexture("\\media\\button.tga")
			end
			
			-- create scripts
			if t[i].scripts then
				for y=1,#t[i].scripts do
					FrameX.tframes[i]:SetScript(t[i].scripts[y].event, t[i].scripts[y].func)
				end
			end
		else
			assert(false, L_FRAMEX_ERROR .. " #" .. i) -- Error
		end
	end
end

function FrameX:setVisible( visible )
	if visible then
		for i = 1, #FrameX.tframes do
			FrameX.tframes[i]:Show()
		end
	else
		for i = 1, #FrameX.tframes do
			FrameX.tframes[i]:Hide()
		end
	end
end

function FrameX:setFrameVisible(i, s)
	if s then
		self.tframes[i]:Show()
	else
		self.tframes[i]:Hide()
	end
end

function FrameX:updateFrames(total, cur, minus)
	total = total + minus
	cur = cur + minus
	for i = 1, table.getn(self.tframes) do
			--print("Pass: "..i, "Object: "..self.tframes[i].name)
		if not self.tframes[i].root or self.tframes[i].root == false then
			local rgba = {}
			local rgbashaded = {}
			
			-- parent info or not
			if self.tframes[i].parentrgba and self.tframes[i].parentrgba == true then
				local id = self:findFrame(self.tframes[i].parentname)
				if id then
					--print(self.tframes[i].name.." is taking child 1")
					rgba = self.tframes[id].childrgba
					rgbashaded = self.tframes[id].childrgbashaded
				else
					--print(self.tframes[i].name.." is taking self 1")
					rgba = self.tframes[id].rgba
					rgbashaded = self.tframes[id].rgbashaded
				end
			else
				--print(self.tframes[i].name.." has no parent")
				rgba = self.tframes[i].rgba
				rgbashaded = self.tframes[i].rgbashaded
			end

			-- loop through frames
			if i <= cur then
				self.tframes[i]:Show()
				self.tframes[i]:SetBackdropColor(rgba.r/255, rgba.g/255, rgba.b/255, rgba.a)
			elseif i <= total then
				self.tframes[i]:Show()
				self.tframes[i]:SetBackdropColor(rgbashaded.r/255, rgbashaded.g/255, rgbashaded.b/255, rgbashaded.a)
			elseif i > total then
				--print(self.tframes[i].name.." hiding")
				self.tframes[i]:Hide()
			end
		end
	end
end

function FrameX:loadOptions()
	return self.GJBPoptions
end

function FrameX:saveOptions(t)
	self.GJBPoptions = t
end

function FrameX:setPosition(a, x, y)
	FrameX.tframes[1]:SetPoint(a, p, nil, x, y)
end

--local function OnUpdate()
	--FrameX:updateFrames()
--end

--FrameX.frame:SetScript("OnUpdate", OnUpdate);